// query et traitement des contenus

import { defineStore } from 'pinia';
import REST from '../api/rest-axios';

export const useContentStore = defineStore('content', {
    state: () => ({
        contentType: '',
        pageTitle: '',
        content: {
            contentTitle: '',
            coordinates: {},
            adresse: {},
            etape_number: '',
            couleur: '',
            dates: {},
            previous : {},
            next: {},
            vignette: {},
            parties: [],

            intro: '',
            partenaires: [],
            gouvernances: [],
        },
        loading: false,
        error: null,
    }),
    actions: {
        async fetchContentData(path) {
            this.resetStore(false);
            const contentTypes = [ 'etape', 'static', 'gouvernance', 'partenaire' ];
            try {
                const findContentByPath = async (contentTypes, path) => {
                    for (let type of contentTypes) {
                        const response = await REST.get(`/jsonapi/node/${type}/`);
                        
                        const content = response.data.data.find(content => 
                            content.attributes.metatag.some(tag => 
                                tag.tag === "link" && tag.attributes.href === path
                            )
                        );
                    
                        if (content) {
                            return {
                                contentType: type,
                                rawContent: content,
                            };
                        }
                    
                        // Handle special case for governance/partners (multiple items per page)
                        const pageRequested = window.location.href.split('/').pop().replace(/s?$/, '');
                        if (type === pageRequested
                            || (type === 'gouvernance' && pageRequested === 'contact')
                        ) {
                            return {
                                contentType: type,
                                rawContent: response.data.data,
                            };
                        }
                    }
                    return null;
                };

                const { contentType, rawContent } = await findContentByPath(contentTypes, path);
                
                this.contentType = contentType;

                // console.log(`current type: ${contentType}`);

                

                if (this.contentType !== 'gouvernance' && this.contentType !== 'partenaire') {                    
                    // pageTitle
                    for (let tag of rawContent.attributes.metatag) {
                        if (tag.tag === "meta") {
                            this.pageTitle = tag.attributes.content;
                            break;
                        }
                    }
                    // contentTitle
                    this.content.contentTitle = rawContent.attributes.title;

                    // vignette
                    const vignetteFetch = await this.fetchFromRelationships('field_vignette', rawContent.relationships);                    
                    if (vignetteFetch) {
                        this.content.vignette = {
                            url: vignetteFetch.attributes.uri.url,
                            alt: rawContent.relationships.field_vignette.data.meta.alt
                        };
                    }

                    if (contentType === 'etape') {
                        // coordinates
                        this.content.coordinates = {
                            lat: rawContent.attributes.field_geofield.lat,
                            lon: rawContent.attributes.field_geofield.lon,
                        };
                        // adresse
                        this.content.adresse = rawContent.attributes.field_adresse;
                        // étape number
                        this.content.etape_number = rawContent.attributes.field_arret_numero;
                        // couleur
                        this.content.couleur = rawContent.attributes.field_couleur;
                        // dates
                        this.content.dates = {
                            start: this.getCleanDate(rawContent.attributes.field_dates.value),
                            end: this.getCleanDate(rawContent.attributes.field_dates.end_value),
                        }
                        // previous / next
                        await this.getRelatedEtape('previous', path);
                        await this.getRelatedEtape('next', path);
                    }

                    // parties
                    const fieldParties = contentType === 'etape' ? 'field_parties' : 'field_parties_static';
                    const partiesFetch = await this.fetchFromRelationships(fieldParties, rawContent.relationships);

                    if (partiesFetch) {
                        this.content.parties = [];
                        for (let partie of partiesFetch) {
                            const partieType = partie.type.replace(/^paragraph--/, "");
                            let partieContent = {
                                type: partieType,
                            };

                            switch (partieType) {
                                case 'carte_sensible':
                                    const carteSensibleFetch = await this.fetchFromRelationships('field_image_carte', partie.relationships);
                                    if (carteSensibleFetch) {
                                        partieContent.carteSensible = {
                                            url: carteSensibleFetch.attributes.uri.url,
                                            alt: partie.relationships.field_image_carte.data.meta.alt,
                                        };
                                    }
                                    break;
                                case 'titre_texte':
                                    partieContent.titre = partie.attributes.field_titre;
                                    partieContent.texte = partie.attributes.field_texte.value;
                                    break;
                                case 'chiffres_cles':
                                    const chiffresClesFetch = await this.fetchFromRelationships('field_chiffres_clefs', partie.relationships);
                                    if (chiffresClesFetch) {
                                        partieContent.chiffresCles = [];
                                        for (let chiffre of chiffresClesFetch) {
                                            partieContent.chiffresCles.push({
                                                chiffre: chiffre.attributes.field_chiffre,
                                                description: chiffre.attributes.field_description,
                                            });
                                        }
                                    }
                                    break;
                                case 'diaporama':
                                    const diaporamaFetch = await this.fetchFromRelationships('field_diaporama', partie.relationships);
                                    if (diaporamaFetch) {
                                        partieContent.diaporama = [];
                                        for (let [index, image] of diaporamaFetch.entries()) {
                                            partieContent.diaporama.push({
                                                url: image.attributes.uri.url,
                                                alt: partie.relationships.field_diaporama.data[index].meta.alt,
                                            });
                                        }
                                    }
                                    break;
                                case 'entretien':
                                    partieContent.entretien = {};
                                    partieContent.entretien.titre = partie.attributes.field_titre;
                                    const personnesFetch = await this.fetchFromRelationships('field_personne_s', partie.relationships);
                                    const questionsReponsesFetch = await this.fetchFromRelationships('field_questions_reponses', partie.relationships);
                                    if (personnesFetch && questionsReponsesFetch) {
                                        partieContent.entretien.personnes = [];
                                        for (let personne of personnesFetch) {
                                            const portraitFetch = await this.fetchFromRelationships('field_portrait', personne.relationships);
                                            if (portraitFetch) {
                                                partieContent.entretien.personnes.push({
                                                    portrait: portraitFetch.attributes.uri.url,
                                                    alt: personne.relationships.field_portrait.data.meta.alt,
                                                    description: personne.attributes.field_description,
                                                });
                                            }
                                        }
                                        partieContent.entretien.questionsReponses = [];
                                        for (let qr of questionsReponsesFetch) {
                                            partieContent.entretien.questionsReponses.push({
                                                question: qr.attributes.field_question,
                                                reponse: qr.attributes.field_reponse.value,
                                            });
                                        }
                                    }
                                    break;
                                case 'exergue':
                                    partieContent.exergue = partie.attributes.field_texte_exergue.value;
                                    break;
                                case 'video':
                                    partieContent.videos = [];
                                    for (let video of partie.attributes.field_videos) {
                                        const videoId = video.split('?v=')[1];
                                        const videoUrl = `https://www.youtube.com/embed/${videoId}`;
                                        partieContent.videos.push(videoUrl);
                                    }
                                    break;
                            }
                            this.content.parties.push(partieContent);
                        }
                    }
                } else {
                    // pages gouvernance (contact) et partenaire
                    // ont plusieurs items par pages
                    const intro = await REST.get(`/jsonapi/config_pages/intro_${this.contentType}/`);
                    const introContent = intro.data.data[0];

                    this.pageTitle = 
                        `${introContent.attributes.field_titre} ${introContent.attributes.metatag.find(tag => tag.tag === "meta")?.attributes.content}`;

                    this.content.contentTitle = introContent.attributes.field_titre;
                    this.content.intro = introContent.attributes.field_intro.value;

                    const multiItemPageArray = [];

                    if (this.contentType === 'partenaire') {
                        for (let item of rawContent) {       
                            const logoFetch = await REST.get(item.relationships.field_logo.links.related.href);
    
                            multiItemPageArray.push({
                                title: item.attributes.title,
                                description: item.attributes.body.value,
                                weight: item.attributes.field_poid,
                                link_url: item.attributes.field_lien.uri ,
                                logo_alt: item.relationships.field_logo.data.meta.alt,
                                logo_url: logoFetch.data.data.attributes.uri.url,
                            });
                        }
                    } else if (this.contentType === 'gouvernance') {
                        for (let item of rawContent) {
                            const personnesFetch = await REST.get(item.relationships.field_personne_s.links.related.href);
                            let personnes = [];
                            for (let personne of personnesFetch.data.data) {
                                const portraitFetch = await REST.get(personne.relationships.field_portrait.links.related.href);
                                personnes.push({
                                    nom: personne.attributes.field_nom,
                                    prenom: personne.attributes.field_prenom,
                                    description: personne.attributes.field_description,
                                    photo_meta: personne.relationships.field_portrait.data?.meta.alt,
                                    photo_url: portraitFetch.data.data?.attributes.uri.url,
                                });
                            }
                            multiItemPageArray.push({
                                title: item.attributes.title,
                                weight: item.attributes.field_poid,
                                personnes: personnes,
                            });
                        }
                    }

                    this.content[`${this.contentType}s`] = multiItemPageArray;
                    
                    console.log(this.content);
                }
            } catch (error) {
                this.error = 'Failed to fetch data';
                console.error('Issue with getNodeData', error);
            } finally {
                this.loading = false;
            }
        },
        getCleanDate(date) {
            return {
              d: date.split('-')[2],
              m: new Intl.DateTimeFormat('fr-FR', { month: 'long' }).format(new Date(date)),
              y: date.split('-')[0],
            }
        },
        async getRelatedEtape(direction, path) {
             const getRelatedEtapeContent = async (relatedEtapeData) => {
                if (relatedEtapeData) {
                    const vignetteFetch = await REST.get(relatedEtapeData.relationships.field_vignette.links.related.href);
                    if (vignetteFetch.data.data) {
                        this.content[direction] = {
                            url: relatedEtapeData.attributes.metatag.find(tag => tag.tag === "link")?.attributes.href,
                            couleur: relatedEtapeData.attributes.field_couleur,
                            title: relatedEtapeData.attributes.title,
                            postalCode: relatedEtapeData.attributes.field_adresse.postal_code,
                            dates: {
                                start: this.getCleanDate(relatedEtapeData.attributes.field_dates.value),
                                end: this.getCleanDate(relatedEtapeData.attributes.field_dates.end_value),
                            },
                            vignette: {
                                url: vignetteFetch.data.data.attributes.uri.url,
                                alt: relatedEtapeData.relationships.field_vignette.data.meta.alt,
                            },
                        }
                    }
                    
                }
            }

            const allEtapesData = await REST.get('/jsonapi/views/etapes/block_1/');
            for (let [index, etape] of allEtapesData.data.data.entries()) {
                if (etape.attributes.metatag.some(tag => 
                    tag.tag === "link" && tag.attributes.href === path
                )) {
                    const relatedEtapeIndex = direction === 'next' ? index + 1 : index - 1;
                    await getRelatedEtapeContent(allEtapesData.data.data[relatedEtapeIndex]);
                }
            }
        },
        async fetchFromRelationships(field, relationships) {
            if (relationships[field].links) {
              try {
                const contentLink = relationships[field].links.related.href;
                const contentFetch = await REST.get(contentLink);               
                return contentFetch.data.data;
              } catch (error) {
                this.error = 'Failed to fetch data';
                console.error('Issue with getNodeData', error);
              }
            }
        },
        resetStore(forFrontDisplay) {
            this.contentType = '';
            this.pageTitle = '';
            this.content = {};
            this.loading = !forFrontDisplay;
            this.error = null;
        }
    },
});
